from itemloaders.processors import SelectJmes

test_list_equals = {
    "simple": ("foo.bar", {"foo": {"bar": "baz"}}, "baz"),
    "invalid": ("foo.bar.baz", {"foo": {"bar": "baz"}}, None),
    "top_level": ("foo", {"foo": {"bar": "baz"}}, {"bar": "baz"}),
    "double_vs_single_quote_string": ("foo.bar", {"foo": {"bar": "baz"}}, "baz"),
    "dict": (
        "foo.bar[*].name",
        {"foo": {"bar": [{"name": "one"}, {"name": "two"}]}},
        ["one", "two"],
    ),
    "list": ("[1]", [1, 2], 2),
}


def test_output():
    for key, value in test_list_equals.items():
        expr, test_list, expected = value
        test = SelectJmes(expr)(test_list)
        assert test == expected, f"test {key!r} got {test} expected {expected}"



from itemloaders.utils import arg_to_iter


def test_arg_to_iter():
    assert hasattr(arg_to_iter(None), "__iter__")
    assert hasattr(arg_to_iter(100), "__iter__")
    assert hasattr(arg_to_iter("lala"), "__iter__")
    assert hasattr(arg_to_iter([1, 2, 3]), "__iter__")
    assert hasattr(arg_to_iter(letter for letter in "abcd"), "__iter__")

    assert list(arg_to_iter(None)) == []
    assert list(arg_to_iter("lala")) == ["lala"]
    assert list(arg_to_iter(100)) == [100]
    assert list(arg_to_iter(letter for letter in "abc")) == ["a", "b", "c"]
    assert list(arg_to_iter([1, 2, 3])) == [1, 2, 3]
    assert list(arg_to_iter({"a": 1})) == [{"a": 1}]



"""Common functions used in Item Loaders code"""

from __future__ import annotations

from functools import partial
from typing import TYPE_CHECKING, Any

from itemloaders.utils import get_func_args

if TYPE_CHECKING:
    from collections.abc import Callable, MutableMapping


def wrap_loader_context(
    function: Callable[..., Any], context: MutableMapping[str, Any]
) -> Callable[..., Any]:
    """Wrap functions that receive loader_context to contain the context
    "pre-loaded" and expose a interface that receives only one argument
    """
    if "loader_context" in get_func_args(function):
        return partial(function, loader_context=context)
    return function



import re
from unittest.mock import MagicMock

import pytest
from parsel import Selector

from itemloaders import ItemLoader
from itemloaders.processors import MapCompose, TakeFirst


class CustomItemLoader(ItemLoader):
    name_in = MapCompose(lambda v: v.title())


class TestSelectorItemLoader:
    selector = Selector(
        text="""
    <html>
    <body>
    <div id="id">marta</div>
    <p>paragraph</p>
    <a href="http://www.scrapy.org">homepage</a>
    <img src="/images/logo.png" width="244" height="65" alt="Scrapy">
    </body>
    </html>
    """
    )

    jmes_selector = Selector(
        text="""
    {
      "name": "marta",
      "description": "paragraph",
      "website": {
        "url": "http://www.scrapy.org",
        "name": "homepage"
      },
      "logo": "/images/logo.png"
    }
    """
    )

    def test_init_method(self):
        loader = CustomItemLoader()
        assert loader.selector is None

    def test_init_method_errors(self):
        loader = CustomItemLoader()
        with pytest.raises(RuntimeError):
            loader.add_xpath("url", "//a/@href")
        with pytest.raises(RuntimeError):
            loader.replace_xpath("url", "//a/@href")
        with pytest.raises(RuntimeError):
            loader.get_xpath("//a/@href")
        with pytest.raises(RuntimeError):
            loader.add_css("name", "#name::text")
        with pytest.raises(RuntimeError):
            loader.replace_css("name", "#name::text")
        with pytest.raises(RuntimeError):
            loader.get_css("#name::text")

    def test_init_method_with_selector(self):
        loader = CustomItemLoader(selector=self.selector)
        assert loader.selector

        loader.add_xpath("name", "//div/text()")
        assert loader.get_output_value("name") == ["Marta"]

    def test_init_method_with_selector_css(self):
        loader = CustomItemLoader(selector=self.selector)
        assert loader.selector

        loader.add_css("name", "div::text")
        assert loader.get_output_value("name") == ["Marta"]

        loader.add_css("url", "a::attr(href)")
        assert loader.get_output_value("url") == ["http://www.scrapy.org"]

        # combining/accumulating CSS selectors and XPath expressions
        loader.add_xpath("name", "//div/text()")
        assert loader.get_output_value("name") == ["Marta", "Marta"]

        loader.add_xpath("url", "//img/@src")
        assert loader.get_output_value("url") == [
            "http://www.scrapy.org",
            "/images/logo.png",
        ]

    def test_add_xpath_re(self):
        loader = CustomItemLoader(selector=self.selector)
        loader.add_xpath("name", "//div/text()", re="ma")
        assert loader.get_output_value("name") == ["Ma"]

        loader = CustomItemLoader(selector=self.selector)
        loader.add_xpath("name", "//div/text()", re=re.compile("ma"))
        assert loader.get_output_value("name") == ["Ma"]

    def test_add_xpath_variables(self):
        loader = CustomItemLoader(selector=self.selector)
        loader.add_xpath("name", "id($id)/text()", id="id")
        assert loader.get_output_value("name") == ["Marta"]
        loader = CustomItemLoader(selector=self.selector)
        loader.add_xpath("name", "id($id)/text()", id="id2")
        assert loader.get_output_value("name") == []

    def test_replace_xpath(self):
        loader = CustomItemLoader(selector=self.selector)
        assert loader.selector
        loader.add_xpath("name", "//div/text()")
        assert loader.get_output_value("name") == ["Marta"]
        loader.replace_xpath("name", "//p/text()")
        assert loader.get_output_value("name") == ["Paragraph"]

        loader.replace_xpath("name", ["//p/text()", "//div/text()"])
        assert loader.get_output_value("name") == ["Paragraph", "Marta"]

    def test_get_xpath(self):
        loader = CustomItemLoader(selector=self.selector)
        assert loader.get_xpath("//p/text()") == ["paragraph"]
        assert loader.get_xpath("//p/text()", TakeFirst()) == "paragraph"
        assert loader.get_xpath("//p/text()", TakeFirst(), re="pa") == "pa"

        assert loader.get_xpath(["//p/text()", "//div/text()"]) == [
            "paragraph",
            "marta",
        ]

    def test_replace_xpath_multi_fields(self):
        loader = CustomItemLoader(selector=self.selector)
        loader.add_xpath(None, "//div/text()", TakeFirst(), lambda x: {"name": x})
        assert loader.get_output_value("name") == ["Marta"]
        loader.replace_xpath(None, "//p/text()", TakeFirst(), lambda x: {"name": x})
        assert loader.get_output_value("name") == ["Paragraph"]

    def test_replace_xpath_re(self):
        loader = CustomItemLoader(selector=self.selector)
        assert loader.selector
        loader.add_xpath("name", "//div/text()")
        assert loader.get_output_value("name") == ["Marta"]
        loader.replace_xpath("name", "//div/text()", re="ma")
        assert loader.get_output_value("name") == ["Ma"]

    def test_add_css_re(self):
        loader = CustomItemLoader(selector=self.selector)
        loader.add_css("name", "div::text", re="ma")
        assert loader.get_output_value("name") == ["Ma"]

        loader.add_css("url", "a::attr(href)", re="http://(.+)")
        assert loader.get_output_value("url") == ["www.scrapy.org"]

        loader = CustomItemLoader(selector=self.selector)
        loader.add_css("name", "div::text", re=re.compile("ma"))
        assert loader.get_output_value("name") == ["Ma"]

        loader.add_css("url", "a::attr(href)", re=re.compile("http://(.+)"))
        assert loader.get_output_value("url") == ["www.scrapy.org"]

    def test_replace_css(self):
        loader = CustomItemLoader(selector=self.selector)
        assert loader.selector
        loader.add_css("name", "div::text")
        assert loader.get_output_value("name") == ["Marta"]
        loader.replace_css("name", "p::text")
        assert loader.get_output_value("name") == ["Paragraph"]

        loader.replace_css("name", ["p::text", "div::text"])
        assert loader.get_output_value("name") == ["Paragraph", "Marta"]

        loader.add_css("url", "a::attr(href)", re="http://(.+)")
        assert loader.get_output_value("url") == ["www.scrapy.org"]
        loader.replace_css("url", "img::attr(src)")
        assert loader.get_output_value("url") == ["/images/logo.png"]

    def test_get_css(self):
        loader = CustomItemLoader(selector=self.selector)
        assert loader.get_css("p::text") == ["paragraph"]
        assert loader.get_css("p::text", TakeFirst()) == "paragraph"
        assert loader.get_css("p::text", TakeFirst(), re="pa") == "pa"

        assert loader.get_css(["p::text", "div::text"]) == ["paragraph", "marta"]
        assert loader.get_css(["a::attr(href)", "img::attr(src)"]) == [
            "http://www.scrapy.org",
            "/images/logo.png",
        ]

    def test_replace_css_multi_fields(self):
        loader = CustomItemLoader(selector=self.selector)
        loader.add_css(None, "div::text", TakeFirst(), lambda x: {"name": x})
        assert loader.get_output_value("name") == ["Marta"]
        loader.replace_css(None, "p::text", TakeFirst(), lambda x: {"name": x})
        assert loader.get_output_value("name") == ["Paragraph"]

        loader.add_css(None, "a::attr(href)", TakeFirst(), lambda x: {"url": x})
        assert loader.get_output_value("url") == ["http://www.scrapy.org"]
        loader.replace_css(None, "img::attr(src)", TakeFirst(), lambda x: {"url": x})
        assert loader.get_output_value("url") == ["/images/logo.png"]

    def test_replace_css_re(self):
        loader = CustomItemLoader(selector=self.selector)
        assert loader.selector
        loader.add_css("url", "a::attr(href)")
        assert loader.get_output_value("url") == ["http://www.scrapy.org"]
        loader.replace_css("url", "a::attr(href)", re=r"http://www\.(.+)")
        assert loader.get_output_value("url") == ["scrapy.org"]

    def test_jmes_not_installed(self):
        selector = MagicMock(spec=Selector)
        del selector.jmespath
        loader = CustomItemLoader(selector=selector)
        with pytest.raises(AttributeError) as err:
            loader.add_jmes("name", "name", re="ma")

        assert (
            str(err.value) == "Please install parsel >= 1.8.1 to get JMESPath support"
        )

    def test_add_jmes_re(self):
        loader = CustomItemLoader(selector=self.jmes_selector)
        loader.add_jmes("name", "name", re="ma")
        assert loader.get_output_value("name") == ["Ma"]

        loader.add_jmes("url", "website.url", re="http://(.+)")
        assert loader.get_output_value("url") == ["www.scrapy.org"]

        loader = CustomItemLoader(selector=self.jmes_selector)
        loader.add_jmes("name", "name", re=re.compile("ma"))
        assert loader.get_output_value("name") == ["Ma"]

        loader.add_jmes("url", "website.url", re=re.compile("http://(.+)"))
        assert loader.get_output_value("url") == ["www.scrapy.org"]

    def test_get_jmes(self):
        loader = CustomItemLoader(selector=self.jmes_selector)
        assert loader.get_jmes("description") == ["paragraph"]
        assert loader.get_jmes("description", TakeFirst()) == "paragraph"
        assert loader.get_jmes("description", TakeFirst(), re="pa") == "pa"

        assert loader.get_jmes(["description", "name"]) == ["paragraph", "marta"]
        assert loader.get_jmes(["website.url", "logo"]) == [
            "http://www.scrapy.org",
            "/images/logo.png",
        ]

    def test_replace_jmes(self):
        loader = CustomItemLoader(selector=self.jmes_selector)
        assert loader.selector
        loader.add_jmes("name", "name")
        assert loader.get_output_value("name") == ["Marta"]
        loader.replace_jmes("name", "description")
        assert loader.get_output_value("name") == ["Paragraph"]

        loader.replace_jmes("name", ["description", "name"])
        assert loader.get_output_value("name") == ["Paragraph", "Marta"]

        loader.add_jmes("url", "website.url", re="http://(.+)")
        assert loader.get_output_value("url") == ["www.scrapy.org"]
        loader.replace_jmes("url", "logo")
        assert loader.get_output_value("url") == ["/images/logo.png"]

    def test_replace_jmes_multi_fields(self):
        loader = CustomItemLoader(selector=self.jmes_selector)
        loader.add_jmes(None, "name", TakeFirst(), lambda x: {"name": x})
        assert loader.get_output_value("name") == ["Marta"]
        loader.replace_jmes(None, "description", TakeFirst(), lambda x: {"name": x})
        assert loader.get_output_value("name") == ["Paragraph"]

        loader.add_jmes(None, "website.url", TakeFirst(), lambda x: {"url": x})
        assert loader.get_output_value("url") == ["http://www.scrapy.org"]
        loader.replace_jmes(None, "logo", TakeFirst(), lambda x: {"url": x})
        assert loader.get_output_value("url") == ["/images/logo.png"]

    def test_replace_jmes_re(self):
        loader = CustomItemLoader(selector=self.jmes_selector)
        assert loader.selector
        loader.add_jmes("url", "website.url")
        assert loader.get_output_value("url") == ["http://www.scrapy.org"]
        loader.replace_jmes("url", "website.url", re=r"http://www\.(.+)")
        assert loader.get_output_value("url") == ["scrapy.org"]

    def test_fluent_interface(self):
        loader = ItemLoader(selector=self.selector)
        item = (
            loader.add_xpath("name", "//body/text()")
            .replace_xpath("name", "//div/text()")
            .add_css("description", "div::text")
            .replace_css("description", "p::text")
            .add_value("url", "http://example.com")
            .replace_value("url", "http://foo")
            .load_item()
        )
        assert item == {
            "name": ["marta"],
            "description": ["paragraph"],
            "url": ["http://foo"],
        }



from __future__ import annotations

from docutils import nodes
from docutils.parsers.rst.roles import set_classes


def setup(app):
    app.add_role("gh", github_role)


def github_role(
    name,
    rawtext,
    text,
    lineno,
    inliner,
    options: dict[str, str] | None = None,
    content=None,
):
    options = options or {}
    if text.isdigit():
        display_text = f"#{text}"
        url = f"https://github.com/scrapy/itemloaders/issues/{text}"
    else:
        short_commit = text[:7]
        display_text = short_commit
        url = f"https://github.com/scrapy/itemloaders/commit/{short_commit}"

    set_classes(options)
    node = nodes.reference(rawtext, display_text, refuri=url, **options)
    return [node], []



"""
Copy/paste from scrapy source at the moment, to ensure tests are working.
Refactoring to come later
"""

from __future__ import annotations

import inspect
from collections.abc import Callable, Generator, Iterable
from functools import partial
from typing import Any


def arg_to_iter(arg: Any) -> Iterable[Any]:
    """Return an iterable based on *arg*.

    If *arg* is a list, a tuple or a generator, it will be returned as is.

    If *arg* is ``None``, an empty list will be returned.

    If *arg* is anything else, a list will be returned with *arg* as its only
    item, i.e. ``[arg]``.
    """
    if arg is None:
        return []
    if isinstance(arg, (list, tuple, Generator)):
        return arg
    return [arg]


def get_func_args(func: Callable[..., Any], stripself: bool = False) -> list[str]:
    """Return the argument name list of a callable object"""
    if not callable(func):
        raise TypeError(f"func must be callable, got {type(func).__name__!r}")

    args: list[str] = []
    try:
        sig = inspect.signature(func)
    except ValueError:
        return args

    if isinstance(func, partial):
        partial_args = func.args
        partial_kw = func.keywords

        for name, param in sig.parameters.items():
            if param.name in partial_args:
                continue
            if partial_kw and param.name in partial_kw:
                continue
            args.append(name)
    else:
        args = list(sig.parameters)

    if stripself and args and args[0] == "self":
        args = args[1:]
    return args



from __future__ import annotations

from abc import ABC, abstractmethod
from typing import Any

from itemloaders import ItemLoader


class TestInitializationBase(ABC):
    @property
    @abstractmethod
    def item_class(self) -> type[Any]:
        raise NotImplementedError

    def test_keep_single_value(self) -> None:
        """Loaded item should contain values from the initial item"""
        input_item = self.item_class(name="foo")
        il = ItemLoader(item=input_item)
        loaded_item = il.load_item()
        assert isinstance(loaded_item, self.item_class)
        assert dict(loaded_item) == {"name": ["foo"]}

    def test_keep_list(self) -> None:
        """Loaded item should contain values from the initial item"""
        input_item = self.item_class(name=["foo", "bar"])
        il = ItemLoader(item=input_item)
        loaded_item = il.load_item()
        assert isinstance(loaded_item, self.item_class)
        assert dict(loaded_item) == {"name": ["foo", "bar"]}

    def test_add_value_singlevalue_singlevalue(
        self,
    ) -> None:
        """Values added after initialization should be appended"""
        input_item = self.item_class(name="foo")
        il = ItemLoader(item=input_item)
        il.add_value("name", "bar")
        loaded_item = il.load_item()
        assert isinstance(loaded_item, self.item_class)
        assert dict(loaded_item) == {"name": ["foo", "bar"]}

    def test_add_value_singlevalue_list(self) -> None:
        """Values added after initialization should be appended"""
        input_item = self.item_class(name="foo")
        il = ItemLoader(item=input_item)
        il.add_value("name", ["item", "loader"])
        loaded_item = il.load_item()
        assert isinstance(loaded_item, self.item_class)
        assert dict(loaded_item) == {"name": ["foo", "item", "loader"]}

    def test_add_value_list_singlevalue(self) -> None:
        """Values added after initialization should be appended"""
        input_item = self.item_class(name=["foo", "bar"])
        il = ItemLoader(item=input_item)
        il.add_value("name", "qwerty")
        loaded_item = il.load_item()
        assert isinstance(loaded_item, self.item_class)
        assert dict(loaded_item) == {"name": ["foo", "bar", "qwerty"]}

    def test_add_value_list_list(self) -> None:
        """Values added after initialization should be appended"""
        input_item = self.item_class(name=["foo", "bar"])
        il = ItemLoader(item=input_item)
        il.add_value("name", ["item", "loader"])
        loaded_item = il.load_item()
        assert isinstance(loaded_item, self.item_class)
        assert dict(loaded_item) == {"name": ["foo", "bar", "item", "loader"]}

    def test_get_output_value_singlevalue(self) -> None:
        """Getting output value must not remove value from item"""
        input_item = self.item_class(name="foo")
        il = ItemLoader(item=input_item)
        assert il.get_output_value("name") == ["foo"]
        loaded_item = il.load_item()
        assert isinstance(loaded_item, self.item_class)
        assert loaded_item == {"name": ["foo"]}

    def test_get_output_value_list(self) -> None:
        """Getting output value must not remove value from item"""
        input_item = self.item_class(name=["foo", "bar"])
        il = ItemLoader(item=input_item)
        assert il.get_output_value("name") == ["foo", "bar"]
        loaded_item = il.load_item()
        assert isinstance(loaded_item, self.item_class)
        assert loaded_item == {"name": ["foo", "bar"]}

    def test_values_single(self) -> None:
        """Values from initial item must be added to loader._values"""
        input_item = self.item_class(name="foo")
        il = ItemLoader(item=input_item)
        assert il._values.get("name") == ["foo"]

    def test_values_list(self) -> None:
        """Values from initial item must be added to loader._values"""
        input_item = self.item_class(name=["foo", "bar"])
        il = ItemLoader(item=input_item)
        assert il._values.get("name") == ["foo", "bar"]


class InitializationFromDictTest(TestInitializationBase):
    item_class = dict



from __future__ import annotations

import functools
import operator
import platform
import sys
from typing import Any

from itemloaders.utils import get_func_args


def test_get_func_args():
    def f1(a, b, c):
        pass

    def f2(a, b=None, c=None):
        pass

    def f3(a, b=None, *, c=None):
        pass

    class A:
        def __init__(self, a: Any, b: Any, c: Any):
            pass

        def method(self, a, b, c):
            pass

    class Callable:
        def __call__(self, a, b, c):
            pass

    a = A(1, 2, 3)
    cal = Callable()
    partial_f1 = functools.partial(f1, None)
    partial_f2 = functools.partial(f1, b=None)
    partial_f3 = functools.partial(partial_f2, None)

    assert get_func_args(f1) == ["a", "b", "c"]
    assert get_func_args(f2) == ["a", "b", "c"]
    assert get_func_args(f3) == ["a", "b", "c"]
    assert get_func_args(A) == ["a", "b", "c"]
    assert get_func_args(a.method) == ["a", "b", "c"]
    assert get_func_args(partial_f1) == ["b", "c"]
    assert get_func_args(partial_f2) == ["a", "c"]
    assert get_func_args(partial_f3) == ["c"]
    assert get_func_args(cal) == ["a", "b", "c"]
    assert get_func_args(object) == []
    assert get_func_args(str.split, stripself=True) == ["sep", "maxsplit"]
    assert get_func_args(" ".join, stripself=True) == ["iterable"]

    if sys.version_info >= (3, 13) or platform.python_implementation() == "PyPy":
        # the correct and correctly extracted signature
        assert get_func_args(operator.itemgetter(2), stripself=True) == ["obj"]
    elif platform.python_implementation() == "CPython":
        # ["args", "kwargs"] is a correct result for the pre-3.13 incorrect function signature
        # [] is an incorrect result on even older CPython (https://github.com/python/cpython/issues/86951)
        assert get_func_args(operator.itemgetter(2), stripself=True) in [
            [],
            ["args", "kwargs"],
        ]



from __future__ import annotations

from typing import Any

import pytest

from itemloaders import ItemLoader


def _test_item(item: Any) -> None:
    il = ItemLoader()
    il.add_value("item_list", item)
    assert il.load_item() == {"item_list": [item]}


def test_attrs():
    try:
        import attr  # noqa: PLC0415
    except ImportError:
        pytest.skip("Cannot import attr")

    @attr.s
    class TestItem:
        foo = attr.ib()

    _test_item(TestItem(foo="bar"))


def test_dataclass():
    try:
        from dataclasses import dataclass  # noqa: PLC0415
    except ImportError:
        pytest.skip("Cannot import dataclasses.dataclass")

    @dataclass
    class TestItem:
        foo: str

    _test_item(TestItem(foo="bar"))


def test_dict():
    _test_item({"foo": "bar"})


def test_scrapy_item():
    try:
        from scrapy import Field, Item  # noqa: PLC0415
    except ImportError:
        pytest.skip("Cannot import Field or Item from scrapy")

    class TestItem(Item):
        foo = Field()

    _test_item(TestItem(foo="bar"))



import pytest

from itemloaders.processors import Compose, Identity, Join, MapCompose, TakeFirst


def test_take_first():
    proc = TakeFirst()
    assert proc([None, "", "hello", "world"]) == "hello"
    assert proc([None, "", 0, "hello", "world"]) == 0


def test_identity():
    proc = Identity()
    assert proc([None, "", "hello", "world"]) == [None, "", "hello", "world"]


def test_join():
    proc = Join()
    with pytest.raises(TypeError):
        proc([None, "", "hello", "world"])
    assert proc(["", "hello", "world"]) == " hello world"
    assert proc(["hello", "world"]) == "hello world"
    assert isinstance(proc(["hello", "world"]), str)


def test_compose():
    proc = Compose(lambda v: v[0], str.upper)
    assert proc(["hello", "world"]) == "HELLO"
    proc = Compose(str.upper)
    assert proc(None) is None
    proc = Compose(str.upper, stop_on_none=False)
    with pytest.raises(
        ValueError,
        match=r"Error in Compose with .* error='TypeError: (descriptor 'upper'|'str' object expected)",
    ):
        proc(None)
    proc = Compose(str.upper, lambda x: x + 1)
    with pytest.raises(
        ValueError,
        match=r"Error in Compose with .* error='TypeError: (can only|unsupported operand)",
    ):
        proc("hello")


def test_mapcompose():
    def filter_world(x):
        return None if x == "world" else x

    proc = MapCompose(filter_world, str.upper)
    assert proc(["hello", "world", "this", "is", "scrapy"]) == [
        "HELLO",
        "THIS",
        "IS",
        "SCRAPY",
    ]
    proc = MapCompose(filter_world, str.upper)
    assert proc(None) == []
    proc = MapCompose(filter_world, str.upper)
    with pytest.raises(
        ValueError,
        match=r"Error in MapCompose with .* error='TypeError: (descriptor 'upper'|'str' object expected)",
    ):
        proc([1])
    proc = MapCompose(filter_world, lambda x: x + 1)
    with pytest.raises(
        ValueError,
        match=r"Error in MapCompose with .* error='TypeError: (can only|unsupported operand)",
    ):
        proc("hello")



from __future__ import annotations

from typing import Any

from itemloaders import ItemLoader
from itemloaders.processors import Compose, Identity, TakeFirst


class TestOutputProcessorDict:
    def test_output_processor(self):
        class TempDict(dict[str, Any]):
            def __init__(self, *args, **kwargs):
                super().__init__(*args, **kwargs)
                self.setdefault("temp", 0.3)

        class TempLoader(ItemLoader):
            default_item_class = TempDict
            default_input_processor = Identity()
            default_output_processor = Compose(TakeFirst())

        loader = TempLoader()
        item = loader.load_item()
        assert isinstance(item, TempDict)
        assert dict(item) == {"temp": 0.3}


class TestOutputProcessorItem:
    def test_output_processor(self):
        class TempLoader(ItemLoader):
            default_input_processor = Identity()
            default_output_processor = Compose(TakeFirst())

        item: dict[str, Any] = {}
        item.setdefault("temp", 0.3)
        loader = TempLoader(item=item)
        item = loader.load_item()
        assert isinstance(item, dict)
        assert dict(item) == {"temp": 0.3}
